close all; clear all; 
clc;
addpath MDH;
addpath code;

options.choice = 'evaluation';

addpath('./utils/');

%% load data
load '../data/LabelmeZeroShot.mat'

%load wiki;
fprintf('Wiki dataset loaded...\n');
rand('seed',1);

%% centralization
fprintf('centralizing data...\n');
%I_te = bsxfun(@minus, I_te, mean(I_tr, 1));
%I_tr = bsxfun(@minus, I_tr, mean(I_tr, 1));
%T_te = bsxfun(@minus, T_te, mean(T_tr, 1));
%T_tr = bsxfun(@minus, T_tr, mean(T_tr, 1));

I_tr = NormalizeFea(I_tr);
I_te = NormalizeFea(I_te);
T_tr = NormalizeFea(T_tr);
T_te = NormalizeFea(T_te);


%% split data
% seen, unseen

% train, query, retrieval
% X1_tr, X1_query, X1_retrieval
% X2_tr, X2_query, X2_retrieval

L_tr_Matrix = L_tr;
L_te_Matrix = L_te;

[a  L_tr] = max(L_tr');
[a  L_te] = max(L_te');

L_tr = L_tr';
L_te = L_te';


run = 10;
map = zeros(run,2);
pre = zeros(run,2);
smap = zeros(run,2);
spre = zeros(run,2);
umap = zeros(run,2);
upre = zeros(run,2);

for i = 1 : run
    
    
classes = randperm(8);
seenClass = classes(3:end);
unseenClass = classes(1:2);
fprintf('Unseen classes:\n');
unseenClass


temp = zeros(length(L_te),1);
for ii=1:2
    temp = temp + ismember(L_te,unseenClass(ii));
end
index_unseen_in_te = find(temp==1);
index_seen_in_te = [1:length(L_te)]';
index_seen_in_te(index_unseen_in_te) = [];
% ------------------------------------------
temp = zeros(length(L_tr),1);
for ii=1:2
    temp = temp + ismember(L_tr,unseenClass(ii));
end
index_unseen_in_tr = find(temp==1);
index_seen_in_tr = [1:length(L_tr)]';
index_seen_in_tr(index_unseen_in_tr) = [];


%%

% train data of seen class. same as retrieal data
X1_SR = I_tr(index_seen_in_tr,:);
X2_SR = T_tr(index_seen_in_tr,:);
L_SR = L_tr_Matrix(index_seen_in_tr,:);

X1_SQ = I_te(index_seen_in_te,:);
X2_SQ = T_te(index_seen_in_te,:);
L_SQ = L_te_Matrix(index_seen_in_te,:);

% data split of unseen data
X1_UR = I_tr(index_unseen_in_tr,:);
X2_UR = T_tr(index_unseen_in_tr,:);
L_UR = L_tr_Matrix(index_unseen_in_tr,:);

X1_UQ = I_te(index_unseen_in_te,:);
X2_UQ = T_te(index_unseen_in_te,:);
L_UQ = L_te_Matrix(index_unseen_in_te,:);


S = labelme_attributes(seenClass,:);
%% set parameters


%% options
value = [1e-5 1e-4 1e-3 1e-2 1e-1 1e0 1e1 1e2 1e3 1e4];

%% 1e0, 1e1, 1e-3, 1e1
options.alpha  = 1e0;
options.beta   = 1e0;

options.lambda1 = 1e1; 
options.lambda2 = 1e1; 

options.gamma  = 1e-3;
options.mu     = 1e-3;

options.xi1    = 1e1;
options.xi2    = 1e1;

% code length
options.k = 5;
options.l = 16;
options.maxItr = 10;

%% 10 times
%for i = 1 : run
    tic
    fprintf('run %d starts...\n', i);
    
    [B, V, P1, P2] = CMAH(X1_SR', X2_SR', L_SR(:,seenClass), S', options);
    
    
    %% Test case 1
    % query    : from unseen classes
    % retrieval: from unseen classes + seen classes
    
    rBX = [sign(X1_UR * P1);B];
    qBX = sign(X1_UQ * P1);
    rBY = [sign(X2_UR * P2);B];
    qBY = sign(X2_UQ * P2);

    rBX = (rBX > 0);
    qBX = (qBX > 0);
    rBY = (rBY > 0);
    qBY = (qBY > 0);
    
    fprintf('\nText-to-Image Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBX,qBY,[L_UR; L_SR] * L_UQ');
    map(i,1) = result.mAP;
    pre(i,1) = result.Precision;
    result
    
    
    fprintf('Image-to-Text Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBY,qBX,[L_UR; L_SR] * L_UQ');
    map(i,2) = result.mAP;
    pre(i,2) = result.Precision;
    result
    

    %% Test case 2
    % query    : from seen classes
    % retrieval: from seen classes
    
    rBX = B;
    qBX = sign(X1_SQ * P1);
    rBY = B;
    qBY = sign(X2_SQ * P2);

    rBX = (rBX > 0);
    qBX = (qBX > 0);
    rBY = (rBY > 0);
    qBY = (qBY > 0);
    
    fprintf('\nSeen Text-to-Image Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBX,qBY,L_SR * L_SQ');
    smap(i,1) = result.mAP;
    spre(i,1) = result.Precision;
    result
    
    
    fprintf('Seen Image-to-Text Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBY,qBX,L_SR *L_SQ');
    smap(i,2) = result.mAP;
    spre(i,2) = result.Precision;
    result
    
    %% Test case 3
    % query    : from unseen classes
    % retrieval: from unseen classes
    
    rBX = sign(X1_UR * P1);
    qBX = sign(X1_UQ * P1);
    rBY = sign(X2_UR * P2);
    qBY = sign(X2_UQ * P2);

    rBX = (rBX > 0);
    qBX = (qBX > 0);
    rBY = (rBY > 0);
    qBY = (qBY > 0);
    
    %fprintf('\nText-to-Image Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBX,qBY,L_UR * L_UQ');
    umap(i,1) = result.mAP;
    upre(i,1) = result.Precision;
    %result
    
    
    %fprintf('Image-to-Text Result:\n');
    [result.mAP, result.Precision, result.Recall, result.F1] = evaluation(rBY,qBX,L_UR * L_UQ');
    umap(i,2) = result.mAP;
    upre(i,2) = result.Precision;
    %result
    
    
    
    toc

end

fprintf('Average Unseen Map & Pre Text-to-Image over %d runs: %.4f & %.4f\n', run, mean(map(:,1)),mean(pre(:,1)));
fprintf('Average Unseen Map & Pre Image-to-Text over %d runs: %.4f & %.4f\n', run, mean(map(:,2)),mean(pre(:,2)));
fprintf('-------------------------------------------------------------\n');
fprintf('Average Seen   Map & Pre Text-to-Image over %d runs: %.4f & %.4f\n', run, mean(smap(:,1)),mean(spre(:,1)));
fprintf('Average Seen   Map & Pre Image-to-Text over %d runs: %.4f & %.4f\n', run, mean(smap(:,2)),mean(spre(:,2)));
fprintf('-------------------------------------------------------------\n');
fprintf('Average U to U Map & Pre Text-to-Image over %d runs: %.4f & %.4f\n', run, mean(umap(:,1)),mean(upre(:,1)));
fprintf('Average U to U Map & Pre Image-to-Text over %d runs: %.4f & %.4f\n', run, mean(umap(:,2)),mean(upre(:,2)));

%fprintf('Average Pre Text-to-Image over %d runs: %.4f\n', run, mean(pre(:,1)));
%fprintf('Average Pre Image-to-Text over %d runs: %.4f\n', run, mean(pre(:,2)));


