function [B, V, P1, P2] = CMAH(X1, X2, Y, S, options)

% ---------- Argument defaults ----------
if ~isfield(options,'debug') || isempty(options.debug)
    debug=1;
else
    debug = options.debug;
end


% label matrix N x c
if isvector(Y) 
    Y = sparse(1:length(Y), double(Y), 1); Y = full(Y);    Y=Y';
end
[n, c] = size(Y);
d1 = size(X1,1);
d2 = size(X2,1);
[a, ~] = size(S);

% initial W,P,B,R
V = rand(options.l,a);
randn('seed',2);
P1 = rand(d1,options.l);
P2 = rand(d2,options.l);
B=sign(randn(n,options.l));

% compute Laplacian matrix
addpath code\;

opts.k = options.k;

L1 = computeLaplacian(X1',opts);
L2 = computeLaplacian(X2',opts);

i = 0;

X1 = double(X1);
X2 = double(X2);

XLX1 = X1*L1*X1';
XX1 = X1*X1';

XLX2 = X2*L2*X2';
XX2 = X2*X2';

SS = S*S';
while i < options.maxItr    
    i=i+1;  
    if debug,fprintf('Iteration  %03d: ',i);end

    % 1. Update P
    P1 = (options.alpha * XLX1 + options.lambda1 * XX1 + options.xi1 * eye(d1)) \ ( options.alpha * X1 * B);
    P2 = (options.beta  * XLX2 + options.lambda2 * XX2 + options.xi2 * eye(d2)) \ ( options.beta  * X2 * B);
    
    % 2. Update V
    V = (B'*B + options.gamma * eye(options.l)) \ (B' * Y * S') / (SS + options.mu * eye(a));
    
    % 2. Update B
    R = V * S;
    Q = V * S * Y' + options.alpha * P1' * X1 + options.beta * P2' * X2;
    Z = B;
    for time = 1
        for k = 1 : size(Z,2)
            Zk = Z; Zk(:,k) = [];
            Rkk = R(k,:); Rk = R; Rk(k,:) = [];
            Vkk = V(k,:); Vk = V; Vk(k,:) = [];
            Z(:,k) = sign(   Q(k,:)' - Zk * Rk * Rkk' - options.beta * Zk * Vk * Vkk'  );
        end
    end
    B = Z;
    

    if debug
        obj_val(i)=compute_CMAH_Cost(X1, X2, P1, P2, B, V, S, Y, XLX1, XLX2, options);
        fprintf('cost=%g\n',obj_val(i));
    end
end
if debug
    plot(obj_val,'-o');
    
    
title('Convergence on Pascal VOC','fontsize',12);
xlabel('#Iteration','fontsize',12);
ylabel('Objective Value','fontsize',12);

%xlim([0.75 4.25]);
%ylim([0.3 0.5]);

set(gcf,'unit','centimeters','position',[0 0 11 9]); % 12 ,7 
set(gca,'FontSize',15);

end